# TASK multiall.cl
# MULTI-SLIT DATA PROCESSING
# THIS PROCEDURE EXTRACTS INDIVIDUAL 2D SPECTRUM FROM AN IMAGE WITH 
# MULTIPLE 2D SPECTRA
# FOR EACH OF THE SLIT POSITIONS READ FROM FILE:
#     Step 1: bias subtraction (full image subraster - full bias subraster)
#     Step 2: flat computation with task response
#     Step 3: flat division 
#     Step 4: Sky spectrum subtraction (background)
#     Step 5: Extraction of corresponding wavelength calib image data
#     Step 6: Assign reference calibration spectrum (refspec)
#     Step 7: Wavelength calibration (dispcor)
#     Step 8: display and splot
# version 1: 15 April 1993
# OLIVIER LE FEVRE, CFHT

procedure multiall (spec_ima,spec_tot,slit_pos,slit_ns,slit_ne,
bias_ima,flat_ima,lambda_cal,lambda_ref,Output)

string spec_ima {prompt="Images w/ 2D spectra"}
string spec_tot {prompt="Sum of 2D spectra images"}
string slit_pos {prompt="File w/ slit positions"}
int slit_ns {1, prompt="Slit number to start"}
int slit_ne {1, prompt="Slit number to end"}
string bias_ima {prompt="Bias image"}
string flat_ima {prompt="Flat image"}
string lambda_cal {prompt="Image w/ wavelength calib"}
string lambda_ref {prompt="Image reference wavelength calib"}
string flux_cal {prompt="sensitivity image for flux calib"}
string Output   {prompt="Output image"}
real lambda0 {prompt="Zero deviation wavelength"}
real lambda1 {prompt="Start wavelength"}
real lambda2 {prompt="End wavelength"}
real disper {prompt="Dispersion in A/pix"}
bool ca {yes, prompt="Subtract bias ?"}
bool cb {yes, prompt="Divide by flat ?"}
bool ch {no, prompt="Divide by slit profile (no flat only) ?"}
int of {prompt="Default order of spline3 for flat fit"}
bool cc {no, prompt="Subtract sky interactively ?"}
bool cd {no, prompt="wavelength calib. interactively ?"}
bool ce {yes, prompt="extract object(s) interactively (>1 per aperture = yes) ?"}
bool cf {yes, prompt="Do you want to flux calibrate ?"}
bool cg {yes, prompt="smooth before extraction ?"}
bool anext {yes, prompt="Go to next object (yes/no) ?", mode="q"}
bool previous {yes, prompt="Redo previous object (yes/no) ?", mode="q"}

struct *crtlist {mode="hl"}
struct *imglist {mode="hl"}
struct *imgdd {mode="hl"}
struct *insertlist {mode="hl"}

begin
	bool anszero,test1
	int ii = 1
	int ij = 1
	int jj = 0
	int xima = 1
	int xima2 = 1
	int i = 0
	int iii = 1
	int lll0 = 0
	int aaa = 0
	int bbb = 0
	int xmin, xmax, ymin, ymax, xcalmin, xcalmax, lmax, xmax2, ymax2
	int x, y, l, i2, i3, diff, ylow, yhigh,xminf,xmaxf
	int y2, ymin2, yima2, llll, lll, ll2
	int xc1,xc2,yc1,yc2,number,num,xi,yi,ymid,yavg,ysup,yrep
	int ll[1000]
	string imagein, list, imfin, imgfile, bias, imageback, imagefin, imagetot
	string zeropos,insertpos, coordins, prepfile,prepfile2,prepfile3
	string imageflaten, imagebias, flat, imageflat, imageflatfin, refcal, imagecal
	string lambda, lambcal, imagecala, imfina, imfinal, ims, imindiv
	string imgff, imgg, imagerest, imfinraw, iflatsmooth, imagezero,imagezerocor
	string imb, imf, img, corfile, imsens, imdisp, imfinala, imdispa
	real disp_cen, ratio, xxm, yym	

	imagein=spec_ima
	imagetot=spec_tot
	list=slit_pos
	imfin=Output
	bias=bias_ima
	flat=flat_ima
	refcal=lambda_ref
	lambcal=lambda_cal
	imsens=flux_cal
	crtlist=list

# set temporary image names

#	imb = mktemp ("ttsb")
#	imf = mktemp ("ttsf")
#	imageback = mktemp ("ttback")
#	lambda = mktemp ("ttlamb")
#	imagecal = imfin//"cal"
#	imdispa = imfin//"l"
#	imfinala = imfin//"fin"
#	imindiv = imfin//"back"
#	imageflat = mktemp ("ttf")
#	imageflaten = mktemp ("ttff")
#	iflatsmooth = mktemp ("ttsmooth")
	prepfile = imfin//"prep"
	prepfile2 = imfin//"prep2"
	prepfile3 = imfin//"prep3"

# start loop on slit positions file

	while (fscan (crtlist, x, y, l) != EOF)  {

	if ( ii < slit_ns || ii > slit_ne) goto slit

toto:
	  print("Slit number = ",ii)

	  if ( cb == yes ) {

	    print("multiprep")

	    multiprep (imagetot,list, ii, flat, prepfile, no, no, yes, ordre=of,
	    lambda0=lambda0, lambda1=lambda1,lambda2=lambda2, disper=disper,
	    cb=yes)

	    } 

	  if ( cb == no ) {

	    if ( ch == yes ) {

	      multiprep (imagetot,list, ii, flat, prepfile, no, no, yes,
	      ordre=of, lambda0=lambda0, lambda1=lambda1,lambda2=lambda2,
	      disper=disper, cb=yes)


	    } 

	    if ( ch == no ) {

	      multiprep (imagetot,list, ii, flat, prepfile, no, no, yes,
	      ordre=of, lambda0=lambda0, lambda1=lambda1,lambda2=lambda2,
	      disper=disper, cb=no)

            }

	  }

	  for (i=1; i<=ii-1; i+=1) {
	    print(i,0,0,0,0,0,0,0,0,0,0,"a ",0,0,0,0,0,"a ",0,0,0,0,0,
	    "a ",0,0,0,0,0,0,0,"a ",0,
	    >> prepfile2)

	  }

	  concatenate(prepfile,prepfile2,append=yes)

	  print("multibatch")

	print(imagein,prepfile2,ii,bias,flat,lambcal,refcal,imfin,
	lambda0,lambda1,disper,ca,cb,ch,cc)
 
	  multibatch (imagein, prepfile2, ii, ii, bias, flat, lambcal, imfin,
	  no, no, yes, 
 	  lambda0=lambda0, lambda1=lambda1,
	  lambda2=lambda2, disper=disper,
	  ca=ca, cb=cb, ch=ch, cc=cc)
#	lambcal,imfin)
 
	  print("multiextract")

	  multiextract (imfin,ii, cd=cd, ce=ce, cf=cf, cg=cg, anext=no,
	  previous=no, slit_nume=ii, 
	  lambda_ref=refcal,flux_cal=imsens)

	  del(prepfile, ver-)
	  del(prepfile2, ver-)

	  if ( anext == yes ) {

	    dir(imfin//ii//"*.imh")

	    } else if ( previous == yes ) {

	      imdel(imfin//ii//"*.imh")
	      imdel(imfin//"cal"//ii//"*.imh")
	      imdel(imfin//"back"//ii//"*.imh")

	      goto toto

	    } else {

	      break

	    }

slit:

	  ii+=1
	  ij=1

	}

	end
	

