# TASK multibatch.cl
# MULTI-SLIT DATA PROCESSING
# FOR EACH OF THE SLIT POSITIONS READ FROM FILE:
# THE CORRECTION INFORMATION IS RETRIEVED FROM FILE CREATED WITH MULTIPREP
# ALL PROCESSING IS DONE AUTOMATICALLY
#     Step 1: bias subtraction (full image subraster - full bias subraster)
#     Step 2: flat computation with task response
#     Step 3: flat division 
#     Step 4: Sky spectrum subtraction (background)
#     Step 5: Extraction of corresponding wavelength calib image data
# OLIVIER LE FEVRE CFHT
# version 1: 19 Dec 1992
# version 1.1: 19 Dec 1992
# version 1.2: 15 April 1993
#     Entry for slit end added

procedure multibatch (spec_ima,slit_pos,slit_ns,slit_ne,bias_ima,
flat_ima,lambda_cal,Output)

string spec_ima {prompt="Images w/ 2D spectra"}
string slit_pos {prompt="Parameter file created by multiprep"}
int slit_ns {1, prompt="Slit number to start"}
int slit_ne {1, prompt="Slit number to end"}
string bias_ima {prompt="Bias image"}
string flat_ima {prompt="Flat image"}
string lambda_cal {prompt="Image w/ wavelength calib"}
string Output   {prompt="Output image"}
real lambda0 {prompt="Zero deviation wavelength"}
real lambda1 {prompt="Start wavelength"}
real lambda2 {prompt="End wavelength"}
real disper {prompt="Dispersion in A/pix"}
bool ca {yes, prompt="Subtract bias ?"}
bool cb {yes, prompt="Divide by flat ?"}
bool ch {no, prompt="Divide by slit profile (no flat only) ?"}
bool cc {no, prompt="Interactive ?"}
bool anext = yes {prompt="Go to next object ?", mode="q"}
bool previous = yes {prompt="Redo previous object (yes/no) ?", mode="q"}
bool stop = yes {prompt="Quit program ?", mode="q"}

struct *crtlist {mode="hl"}
struct *imglist {mode="hl"}
struct *imgdd {mode="hl"}
struct *insertlist {mode="hl"}
struct *imdispp {mode="hl"}
struct *pixcorf {mode="hl"}

begin
	bool anszero,test1, anscor, test2
	int ii = 1
	int ij = 1
	int jj = 0
	int ijj = 0
	int xima = 1
	int xima2 = 1
	int i = 0
	int iii = 1
	int lll0 = 0
	int aaa = 0
	int bbb = 0
	int xmin, xmax, ymin, ymax, xcalmin, xcalmax, lmax, xmax2, ymax2
	int x, y, l, i2, i3, diff, ylow, yhigh,xminf,xmaxf
	int y2, ymin2, yima2, llll, lll, ll2
	int xc1[3],xc2[3],yc1[3],yc2[3],number,num,xi,yi,ymid,yavg,ysup,yrep
	int x1,x2,x3,x4,x5,x6, orderzero, os1, os2, os3, oss
	int ll[1000],of,os
	int xca1,xca2,yca1,yca2,xcb1,xcb2,ycb1,ycb2,xcc1,xcc2,ycc1,ycc2
	int xcor,ycor,xins,yins,xflat1,xflat2
	string imagein, list, imfin, imgfile, bias, imageback 
#	string zeropos1,insertpos1,zeropos2,insertpos2,zeropos3,insertpos3
	string zeropos, insertpos, coordins
	string imageflaten, imagebias, flat, imageflat, imageflatfin
	string imagecal, aa1,aa2,aa3,aa4
	string lambda, lambcal, imagecala, imfina, imfinal, ims, imindiv
	string imgff, imgg, imagerest, imfinraw, iflatsmooth
	string imagezero,imagezerocor,func, funczero, funczero1,funczero2
	string funczero3, idnum, filezerocor
	string imb, imf, img, corfile, imdisp, imfinala, imdispa
	string listflatfin, imgdisp, corfile2, pixcor
#	string imagezerocor1,imagezerocor2,imagezerocor3
	real disp_cen, ratio, xxm, yym,intcor	

	imagein=spec_ima
	list=slit_pos
	imfin=Output
	bias=bias_ima
	flat=flat_ima
	lambcal=lambda_cal
	crtlist=list

# set temporary image names

	imb = mktemp ("ttsb")
	imf = mktemp ("ttsf")
	imageback = mktemp ("ttback")
	lambda = mktemp ("ttlamb")
	imagecal = imfin//"cal"
	imdispa = imfin//"l"
	imfinala = imfin//"fin"
	imindiv = imfin//"back"
	imageflat = mktemp ("ttf")
	imageflaten = mktemp ("ttff")
	iflatsmooth = mktemp ("ttsmooth")

# start loop on slit positions file

	while (fscan (crtlist, x, y, l, xflat1, xflat2, of,number,
	xca1,xca2,yca1,yca2,funczero1,os1,
	xcb1,xcb2,ycb1,ycb2,funczero2,os2,
	xcc1,xcc2,ycc1,ycc2,funczero3,os3,
	x1,x2,x3,x4,x5,x6, func,os) != EOF)  {

	xc1[1]=xca1
	xc2[1]=xca2
	yc1[1]=yca1
	yc2[1]=yca2
	xc1[2]=xcb1
	xc2[2]=xcb2
	yc1[2]=ycb1
	yc2[2]=ycb2
	xc1[3]=xcc1
	xc2[3]=xcc2
	yc1[3]=ycc1
	yc2[3]=ycc2	 

	if ( ii < slit_ns || ii > slit_ne ) goto slit
toto:
	print (ii)
	print (x,y,l)
	print(number, xc1[1],xc2[1],yc1[1],yc2[1],funczero1,os1,
	  xc1[2],xc2[2],yc1[2],yc2[2],funczero2,os2,
	  xc1[3],xc2[3],yc1[3],yc2[3],funczero3,os3,
	  x1,x2,x3,x4,x5,x6, func,os)


	aaa = 0
	bbb = 0
	jj = 0
	ij = 1

	imagecala = imagecal//ii//".0001"
	imfina = imfin//ii//".000*.imh"
	imdisp = imdispa//ii//".000*.imh"
	imfinal = imfinala//ii//".000*.imh"

	corfile = imfin//"lis"
	corfile2 = imfin//"lis2"
	listflatfin = imageflat//"lis"

	zeropos = mktemp ("ttzeropos")
	insertpos = mktemp ("ttins")
	coordins = mktemp ("ttcordins")

	   x=x
	   xmin = x-l/2
	   xmax = x+l/2-1
	   xcalmin = x-2
	   xcalmax = x+2
	   lmax=l-1

# get input images pixel size in Y axis

	   imgets (lambcal,"i_naxis2", value="")
	   ymax=int(imgets.value)

	   print(ymax)

	   xima=xima+1+l

# compute ymin and ymax

	  ylow=y-(lambda0-lambda1)/disper
	  yhigh=y+(lambda2-lambda0)/disper

	  if (ylow < 1 ) {
	    ymin=1
	  } else {
	    ymin=ylow
	  }

	  if ( yhigh > ymax ) {
	    ymax=ymax
	  } else {
	    ymax=yhigh
	  }

	  if (ymax > 2008 ) {
	    ymax=2008
	  }

	  print("ymin,ymax")
	  print(ymin,ymax)

#	print(disp_cen)

	hedit (lambcal,
	"dispaxis", "2", add=yes, delete=no, verify=no, show=no, update=yes)

	hedit (flat,
	"dispaxis", "2", add=yes, delete=no, verify=no, show=no, update=yes)

	print (ca,cb)

	if (ca == yes ) {
	   if (cb == yes ) {

	print("subraster bias")

	  imcopy(bias//"["//xmin//":"//xmax//","//ymin//":"//ymax//"]",
	   imb//ii, verbose=no)

	print("subraster flat")

	  imcopy(flat//"["//xmin//":"//xmax//","//ymin//":"//ymax//"]",
	   imf//ii, verbose=no)

#  subtract bias from flat image

	print("flat - bias")

	   imarith(imf//ii,
	     "-",imb//ii,
	     imageflat//ii, title="", divzero=0., hparams="", 
	     pixtype="real",calctype="real", verbose=no, noact=no)

#  fit flat image to get flat field 

	print("response yes/yes")

        xminf = 10
        xmaxf = xmax-xmin-10

	if ( cc == yes ) {

	   response(imageflat//ii,
             imageflat//ii//"["//xminf//":"//xmaxf//",*]",
	     imageflaten//ii, threshold=INDEF,
	     interactive=yes, sample=""//xflat1//":"//xflat2//"",naverage=3,
	     function="spline3", order=of,, low_reject=3.,
	     high_reject=3.,
	     niterate=5, grow=4., graphics="stdgraph", cursor="")

	} else {

	   response(imageflat//ii,
             imageflat//ii//"["//xminf//":"//xmaxf//",*]",
	     imageflaten//ii, threshold=INDEF,
	     interactive=no, sample=""//xflat1//":"//xflat2//"",naverage=3,
	     function="spline3", order=of,, low_reject=3.,
	     high_reject=3.,
	     niterate=5, grow=4., graphics="stdgraph", cursor="")

	}

	   imarith(imageflat//ii,
	     "/",imageflaten//ii,
	     iflatsmooth//ii, title="", divzero=0., hparams="", 
	     pixtype="real",calctype="real", verbose=no, noact=no)

	  }
	}

	if (ca == yes ) {
	    if (cb == no ) {

	  imcopy(bias//"["//xmin//":"//xmax//","//ymin//":"//ymax//"]",
	   imb//ii, verbose=no)

	    if (ch == yes ) {

	      print("subraster flat")

	      imcopy(flat//"["//xmin//":"//xmax//","//ymin//":"//ymax//"]",
	       imf//ii, verbose=no)

#  subtract bias from flat image

	      print("flat - bias")

	      imarith(imf//ii,
	       "-",imb//ii,
	       imageflat//ii, title="", divzero=0., hparams="", 
	       pixtype="real",calctype="real", verbose=no, noact=no)

#  fit flat image to get flat field 

	      print("response yes/yes")

              xminf = 10
              xmaxf = xmax-xmin-10

	      if ( cc == yes ) {

	      response(imageflat//ii,
               imageflat//ii//"["//xminf//":"//xmaxf//",*]",
	       imageflaten//ii, threshold=INDEF,
	       interactive=yes, sample=""//xflat1//":"//xflat2//"",naverage=3,
	       function="spline3", order=of,, low_reject=3.,
	       high_reject=3.,
	       niterate=5, grow=4., graphics="stdgraph", cursor="")

	      } else { 

	      response(imageflat//ii,
               imageflat//ii//"["//xminf//":"//xmaxf//",*]",
	       imageflaten//ii, threshold=INDEF,
	       interactive=no, sample=""//xflat1//":"//xflat2//"",naverage=3,
	       function="spline3", order=of,, low_reject=3.,
	       high_reject=3.,
	       niterate=5, grow=4., graphics="stdgraph", cursor="")

	      }

	      imarith(imageflat//ii,
	       "/",imageflaten//ii,
	       iflatsmooth//ii, title="", divzero=0., hparams="", 
	       pixtype="real",calctype="real", verbose=no, noact=no)

	      ymid = (ymax-ymin)/2
	      ysup = ymax-ymin-40
	      yavg = ysup - ymid + 1
	      yrep = ymax-ymin+1

	print(ymax,ymin,ymid,yavg)

	      blkavg (imageflaten//ii//"[*,"//ymid//":"//ysup//"]",
	      imageflaten//ii, 1, yavg, 1, 1, 1, 1, 1, option="average")
 
	      blkrep (imageflaten//ii,
	      imageflaten//ii, 1, yrep, 1, 1, 1, 1, 1)
 
	    }
	  }
	}

	if (ca == no ) {
	   if (cb == yes ) {

	print("subraster flat")

	   imcopy (flat//"["//xmin//":"//xmax//","//ymin//":"//ymax//"]",
	   imageflat//ii, verbose=no)

	print("response no/yes")

        xminf = 10
        xmaxf = xmax-xmin-10

	   if ( cc == yes ) { 

	   response(imageflat//ii,
             imageflat//ii//"["//xminf//":"//xmaxf//",*]",
	     imageflaten//ii, threshold=INDEF,
	     interactive=yes, sample=""//xflat1//":"//xflat2//"",naverage=3,
	     function="spline3", order=of,, low_reject=3., high_reject=3.,
	     niterate=5, grow=4., graphics="stdgraph", cursor="")

	    } else {

	   response(imageflat//ii,
             imageflat//ii//"["//xminf//":"//xmaxf//",*]",
	     imageflaten//ii, threshold=INDEF,
	     interactive=no, sample=""//xflat1//":"//xflat2//"",naverage=3,
	     function="spline3", order=of,, low_reject=3., high_reject=3.,
	     niterate=5, grow=4., graphics="stdgraph", cursor="")

	    }

	   imarith(imageflat//ii,
	     "/",imageflaten//ii,
	     iflatsmooth//ii, title="", divzero=0., hparams="", 
	     pixtype="real",calctype="real", verbose=no, noact=no)

	  }
	}

	if ( ca == no ) {
	  if ( cb == no ) {
	    if ( ch == yes ) {

	    print("subraster flat")

	   imcopy (flat//"["//xmin//":"//xmax//","//ymin//":"//ymax//"]",
	    imageflat//ii, verbose=no)

	    print("response no/yes")

            xminf = 10
            xmaxf = xmax-xmin-10

	    if ( cc == yes ) {

	    response(imageflat//ii,
             imageflat//ii//"["//xminf//":"//xmaxf//",*]",
	     imageflaten//ii, threshold=INDEF,
	     interactive=yes, sample=""//xflat1//":"//xflat2//"",naverage=3,
	     function="spline3", order=of,, low_reject=3., high_reject=3.,
	     niterate=5, grow=4., graphics="stdgraph", cursor="")

	    } else {

	    response(imageflat//ii,
             imageflat//ii//"["//xminf//":"//xmaxf//",*]",
	     imageflaten//ii, threshold=INDEF,
	     interactive=no, sample=""//xflat1//":"//xflat2//"",naverage=3,
	     function="spline3", order=of,, low_reject=3., high_reject=3.,
	     niterate=5, grow=4., graphics="stdgraph", cursor="")

	    }

	    imarith(imageflat//ii,
	     "/",imageflaten//ii,
	     iflatsmooth//ii, title="", divzero=0., hparams="", 
	     pixtype="real",calctype="real", verbose=no, noact=no)

	    ymid = (ymax-ymin)/2
	    ysup = ymax - ymin - 40
	    yavg = ysup - ymid + 1
	    yrep = ymax-ymin+1

	print(ymax,ymin,ymid,yavg)

	    blkavg (imageflaten//ii//"[*,"//ymid//":"//ysup//"]",
	    imageflaten//ii, 1, yavg, 1, 1, 1, 1, 1, option="average")
 
	    blkrep (imageflaten//ii,
	    imageflaten//ii, 1, yrep, 1, 1, 1, 1, 1)
 
	    }
	  }
	}

# extract corresponding wavelength calibration image

	print("corresponding ll calib")

	   imcopy(lambcal//"["//xmin//":"//xmax//","//ymin//":"//ymax//"]",
	     imagecal//ii,verbose=no)

# define the input image list

	imgfile = mktemp ("ttfile")
	sections (imagein, option="fullname", > imgfile)
	imglist = imgfile

# start loop on multiple input images

redosky:

	while (fscan (imglist, img) != EOF) {

firstcor:

	print (img)

	hedit (img,
	"dispaxis", "2", add=yes, delete=no, verify=no, show=no, update=yes)
 
	ims = mktemp ("tts")
	imagebias = mktemp ("ttb")
	imageflatfin = mktemp ("ttfff")
	imagerest = mktemp ("ttrest")
	imagezero = mktemp ("ttzero")
	imagezerocor = mktemp ("ttzerocor")
	imfinraw = mktemp ("ttfinraw")

# case correct bias=yes, correct flat = yes

	if (ca == yes ) {
	   if (cb == yes ) {

# subraster image/bias/flat

	print("subraster image")

	imcopy(img//"["//xmin//":"//xmax//","//ymin//":"//ymax//"]",
	ims//ii, verbose=no)

#  subtract bias from spectra image

	print("spectra image - bias")

	   imarith(ims//ii,
	     "-",imb//ii,
	     imagebias//ii, title="", divzero=0., hparams="", 
	     pixtype="real",calctype="real", verbose=no, noact=no)

# divide spectra image by flat field image 

	print("flat yes/yes")

	   imarith(imagebias//ii,
	     "/",imageflaten//ii,
	     imageflatfin//ii, title="", divzero=0., hparams="", 
	     pixtype="real",calctype="real", verbose=no, noact=no)

# compute location of SAOIMAGE image display and display first image for sky ref

	if ( ij == 1 ) {

	  xxm=l
	  yym=ymax-ymin
	  ratio=xxm/yym
	  disp_cen=ratio

	  if ( disp_cen >= (1.-ratio/2) ) {
	    disp_cen=1.-ratio/2
	  }

	   if ( cc == yes ) {

	   display (imageflatfin//ii,
	   1, erase=yes, border_erase=yes, select_frame=yes, repeat=no, 
	   fill=yes, zscale=yes, contrast=0.25, zrange=no, nsample_line=5, 
	   xcenter=disp_cen, ycenter=0.5, xsize=1., ysize=1., xmag=1., 
	   ymag=1., order=0, z1=0., z2=3000.,
	   ztrans="linear", lutfile="")

	   }

	      }
	   }
	}

# case correct bias = no, correct flat = yes

	if ( ca == no ) {
	   if ( cb == yes ) {

	print("flat no/yes")

	   imcopy(img//"["//xmin//":"//xmax//","//ymin//":"//ymax//"]",
	   ims//ii, verbose=no)

	   imarith(ims//ii,
	     "/",imageflaten//ii,
	     imageflatfin//ii, title="", divzero=0., hparams="", 
	     pixtype="real",calctype="real", verbose=no, noact=no)

# compute location of SAOIMAGE image display and display first image for sky ref

	if ( ij == 1 ) {

	  xxm=l
	  yym=ymax-ymin
	  ratio=xxm/yym
	  disp_cen=ratio

	  if ( disp_cen >= (1.-ratio/2) ) {
	    disp_cen=1.-ratio/2
	  }

	   if ( cc == yes ) {

	   display (imageflatfin//ii,
	   1, erase=yes, border_erase=yes, select_frame=yes, repeat=no,
	   fill=yes, zscale=yes, contrast=0.25, zrange=no, nsample_line=5, 
	   xcenter=disp_cen, ycenter=0.5, xsize=1., ysize=1., xmag=1., 
	   ymag=1., order=0, z1=0., z2=3000.,
	   ztrans="linear", lutfile="")

	   }

	      }

	   }
	}

# case correct bias = yes, correct flat = no

	if (ca == yes ) {
	   if (cb == no ) {

	print("subraster image")

	imcopy(img//"["//xmin//":"//xmax//","//ymin//":"//ymax//"]",
	ims//ii, verbose=no)

	print("spectra image - bias")

	   imarith(ims//ii,
	     "-",imb//ii,
	     imageflatfin//ii, title="", divzero=0., hparams="", 
	     pixtype="real",calctype="real", verbose=no, noact=no)

	   if ( ch == yes ) {

	     imarith(imageflatfin//ii,
	       "/",imageflaten//ii,
	       imageflatfin//ii, title="", divzero=0., hparams="", 
	       pixtype="real",calctype="real", verbose=no, noact=no)

	   }

# compute location of SAOIMAGE image display and display first image for sky ref

	if ( ij == 1 ) {

	  xxm=l
	  yym=ymax-ymin
	  ratio=xxm/yym
	  disp_cen=ratio

	  if ( disp_cen >= (1.-ratio/2) ) {
	    disp_cen=1.-ratio/2
	  }

	   if ( cc == yes ) {

	   display (imageflatfin//ii, 1, erase=yes, border_erase=yes,
           select_frame=yes, repeat=no, fill=yes, zscale=no, contrast=0.25, 
	   zrange=no, nsample_line=5, xcenter=disp_cen,
	   ycenter=0.5, xsize=1., ysize=1., xmag=1., ymag=1., order=0, 
	   z1=0., z2=3000., ztrans="linear", lutfile="")

	   }

	      }
	   }
	}

# case correct bias = no, correct flat = no

	if ( ca == no ) {
	   if ( cb == no ) {

	   imcopy (img//"["//xmin//":"//xmax//","//ymin//":"//ymax//"]",
	   imageflatfin//ii, verbose=no)

	   if ( ch == yes ) {

	     imarith(imageflatfin//ii,
	       "/",imageflaten//ii,
	       imageflatfin//ii, title="", divzero=0., hparams="", 
	       pixtype="real",calctype="real", verbose=no, noact=no)

	   }

	print (ij)

# compute location of SAOIMAGE image display and display first image for sky ref

	if ( ij == 1 ) {

	  xxm=l
	  yym=ymax-ymin
	  ratio=xxm/yym
	  disp_cen=ratio

	  if ( disp_cen >= (1.-ratio/2) ) {
	    disp_cen=1.-ratio/2
	  }

	   if ( cc == yes ) {

	   display (imageflatfin//ii, 1, erase=yes, border_erase=yes,
	   select_frame=yes, repeat=no, fill=yes,
	   zscale=no, contrast=0.25, zrange=no, 
	   nsample_line=5, xcenter=disp_cen, ycenter=0.5, xsize=1., 
	   ysize=1., xmag=1., ymag=1., order=0, z1=0., z2=3000.,
	   ztrans="linear", lutfile="")

	   }

	      }
	   }
	}

# prepare list of individual sky corrected images

	print (imagerest//ii, >> corfile2)

	if ( ij != 1 ) {
	
	  print (imagerest//ii, >> corfile)
	  print (imageflatfin//ii, >> listflatfin)

	}

# correct for zero order(s) if necessary

	  if ( ij == 1 ) {

#	    print ("Correct for zero order(s) (yes/no) ?")
#	    scan (anszero)
	    if ( number != 0 ) {
	      anszero=yes
	    } else { anszero=no }

	    test1=anszero

	    if ( test1 == yes ) {

#	      print ("Type number of zero order regions")
#	      scan (number)

	      for (i=1; i<=number; i+=1 ) {

	    if ( i == 1 ) { 
	      oss=os1
	      funczero=funczero1 
	    }

	    if ( i == 2 ) { 
	      oss=os2
	      funczero=funczero2 
	    }

	    if ( i == 3 ) { 
	      oss=os3
	      funczero=funczero3 
	    }

	print(funczero,oss)


#	      print ("Type zero order area: X1, X2, Y1, Y2")
#	      scan (xc1[i],xc2[i],yc1[i],yc2[i])
	      print (xc1[i],xc2[i],yc1[i],yc2[i], >> zeropos//i)
	      print (i,xc1[i],yc1[i], >> insertpos)
	      print (xc1[i],xc2[i],yc1[i],yc2[i])

	print("imcopy") 
	imcopy(imageflatfin//ii//"["//xc1[i]//":"//xc2[i]//","//yc1[i]//":"//yc2[i]//"]",
	imagezero//i, verbose=no)

	print("background")

	      if ( cc == yes ) {

	      background (imagezero//i,
	       imagezerocor//i, axis=1, interactive=yes, sample="*",
	       naverage=1,
	       function=funczero, order=oss, low_reject=2., high_reject=2.,
	       niterate=5, grow=0., graphics="stdgraph", cursor="")

	      } else {

	      background (imagezero//i,
	       imagezerocor//i, axis=1, interactive=no, sample="*",
	       naverage=1,
	       function=funczero, order=oss, low_reject=2., high_reject=2.,
	       niterate=5, grow=0., graphics="stdgraph", cursor="")

	      }

#	     print ("Type function and order for zero order fit")
#	     scan(funczero,orderzero)

#	print(funczero, orderzero)

#	    if ( i == 1 ) { 
#	      os1=orderzero
#	      funczero1=funczero 
#	print(funczero1, os1) }

#	    if ( i == 2 ) { 
#	      os2=orderzero
#	      funczero2=funczero 
#	print(funczero2, os2) }

#	    if ( i == 3 ) { 
#	      os3=orderzero
#	      funczero3=funczero 	
#	print(funczero3, os3) }

	      }

	    for (i=1; i<=number; i+=1 ) {

	print("imedit")

	      imedit (imageflatfin//ii, "", 
	       cursor=zeropos//i, logfile="", display=no, 
	       autodisplay=yes, autosurface=no, aperture="square", radius=2., 
	       search=2., buffer=1., width=2., xorder=2, yorder=2, value=0., 
	       sigma=INDEF, angh=-33., angv=25., 
	    command="display $image 1 erase=$erase fill=yes order=0 >& dev$null",
	       graphics="stdgraph", default="b", fixpix=yes
	   
	      }
	    }
	  } 

	 else {

	  if ( test1 == yes ) {

	    for (i=1; i<=number; i+=1 ) {

	      print (xc1[i],xc2[i],yc1[i],yc2[i])

imcopy(imageflatfin//ii//"["//xc1[i]//":"//xc2[i]//","//yc1[i]//":"//yc2[i]//"]",
	imagezero//i, verbose=no)

	    if ( i == 1 ) { 
	      oss=os1
	      funczero=funczero1 
	    }

	    if ( i == 2 ) { 
	      oss=os2
	      funczero=funczero2 
	    }

	    if ( i == 3 ) { 
	      oss=os3
	      funczero=funczero3 
	    }

	print(funczero,oss)

	      if ( cc == yes ) {

	      background (imagezero//i,
	       imagezerocor//i, axis=1, interactive=yes, sample="*",
	       naverage=1,
	       function=funczero, order=oss, 
	       low_reject=2., high_reject=2.,
	       niterate=5, grow=0., graphics="stdgraph", cursor="")

	      } else { 

	      background (imagezero//i,
	       imagezerocor//i, axis=1, interactive=no, sample="*",
	       naverage=1,
	       function=funczero, order=oss, 
	       low_reject=2., high_reject=2.,
	       niterate=5, grow=0., graphics="stdgraph", cursor="")

	      }

	      }

	print("imedit")

	    for ( i=1; i<=number; i+=1 ) {

	      imedit (imageflatfin//ii, "", 
	       cursor=zeropos//i, logfile="", display=no, 
	       autodisplay=yes, autosurface=no, aperture="square", radius=2., 
	       search=2., buffer=1., width=2., xorder=2, yorder=2, value=0., 
	       sigma=INDEF, angh=-33., angv=25., 
	    command="display $image 1 erase=$erase fill=yes order=0 >& dev$null",
	       graphics="stdgraph", default="b", fixpix=yes)

	      }
	    }
	  }

	print ("zero order correction done")

# sky background correction set-up on image 1

	    if ( ij == 1 ) {

	      if ( cc == yes ) {

	      background (imageflatfin//ii,
	       imagerest//ii, axis=1, interactive=yes, 
	       sample=""//x1//":"//x2//","//x3//":"//x4//","//x5//":"//x6//"",
	       naverage=1,
	       function=func, order=os, low_reject=2., high_reject=2.,
	       niterate=5, grow=0., graphics="stdgraph", cursor="")

	      } else {

	      background (imageflatfin//ii,
	       imagerest//ii, axis=1, interactive=no, 
	       sample=""//x1//":"//x2//","//x3//":"//x4//","//x5//":"//x6//"",
	       naverage=1,
	       function=func, order=os, low_reject=2., high_reject=2.,
	       niterate=5, grow=0., graphics="stdgraph", cursor="")

	      }

#	      print ("Type sample region (x1 x2 x3 x4 x5 x6)")
#	      scan (x1,x2,x3,x4,x5,x6)
#	      print ("Fitting function, order")
#	      scan (func,os) 

# replace corrected zero order region

	    if ( anszero == yes ) {

	      insertlist=insertpos
	      while (fscan (insertlist,i,xi,yi) != EOF) {

	      print (i, xi, yi)
	      del (coordins, ver-)
	      print(xi,yi, >> coordins)

	      iminsert (imagerest//ii,imagezerocor//i, imagerest//ii, 
	       "replace", coordins, offset1=0, offset2=0,
	      xcol="c1", ycol="c2", entry="*")

	      imdel (imagezero//i//".imh")
	      imdel (imagezerocor//i//".imh")


	    }
	  }
	}

# display first corrected image

	if ( ij == 1 ) {

	  disp_cen=disp_cen+ratio

	  if ( disp_cen >= (1.-ratio/2) ) {
	    disp_cen=1.-ratio/2
	  }


	  if ( cc == yes ) {

	  display (imagerest//ii,
	   1, erase=no, border_erase=no, select_frame=yes, repeat=no, 
	   fill=yes, zscale=yes, contrast=0.25, zrange=no, nsample_line=5,
	   xcenter=disp_cen, ycenter=0.5, xsize=1., ysize=1., xmag=1., 
	   ymag=1., order=0, z1=900., z2=3000., ztrans="linear", lutfile="")

	  }

#	  disp_cen=disp_cen-ratio

#	  print("Are you satisfied with correction (yes/no) ?")
#	  scan(anscor)
#	  test2=anscor

#	  if ( test2 == no ) {

#	    imdel(imagerest//ii)
#	    del(zeropos//"*", ver-)
#	    del(insertpos//"*", ver-)
#	    del(corfile, ver-)
#	    del(corfile2, ver-)
#	    del(pixcor//"*", ver-)
#	    imdel(ims//"*.imh")
#	    imdel(imagebias//"*.imh")
#	    imdel(imageflatfin//"*.imh")

#	    goto firstcor

#	  }

	}

	if ( ij != 1) {

# subtract sky background and divide by "smoothed" flat field

	  if ( cc == no) {

	  print("background")

	    background (imageflatfin//ii,
	     imagerest//ii, axis=1, interactive=no,
	     sample=""//x1//":"//x2//","//x3//":"//x4//","//x5//":"//x6//"",
	     naverage=1,
	     function=func, order=os, low_reject=2., high_reject=2.,
	     niterate=5, grow=0., graphics="stdgraph", cursor="")

	  } 

	  else {

	  print("background")

	      background (imageflatfin//ii,
	       imagerest//ii, axis=1, interactive=yes,
	       sample=""//x1//":"//x2//","//x3//":"//x4//","//x5//":"//x6//"",
	       naverage=1,
	       function=func, order=os, low_reject=2., high_reject=2.,
	       niterate=5, grow=0., graphics="stdgraph", cursor="")

	  }

	  if ( anszero == yes ) {

	  insertlist=insertpos

	  while (fscan (insertlist,i,xi,yi) != EOF) {

	    print (i, xi, yi)
	    del (coordins, ver-)
	    print (xi,yi, >> coordins)

	    iminsert (imagerest//ii, imagezerocor//i, imagerest//ii, 
	    "replace", coordins, offset1=0, offset2=0,
	    xcol="c1", ycol="c2", entry="*")

	    imdel (imagezero//i//".imh")
	    imdel(imagezerocor//i)

	    }
	  }


	  disp_cen=disp_cen+ratio

	  if ( disp_cen >= (1.-ratio/2) ) {
	    disp_cen=1.-ratio/2
	  }

	if ( cc == yes ) {

	display (imagerest//ii,
	   1, erase=no, border_erase=no, select_frame=yes, repeat=no, 
	   fill=yes, zscale=yes, contrast=0.25, zrange=no, nsample_line=5,
	   xcenter=disp_cen, ycenter=0.5, xsize=1., ysize=1., xmag=1., 
	   ymag=1., order=0, z1=900., z2=3000., ztrans="linear", lutfile="")

	}

	}

	ij+=1

# end scanning through input images: all the input images are now sky corrected

	}
	

# compute average 2D spectrum from background corrected 2D spectra

	   imstack ("@"//corfile2,imindiv//ii, title="*", pixtype="*")
 
	   imcombine ("@"//corfile2, imfinraw//ii, plfile="", sigma="", 
	   logfile="STDOUT", combine="median",
	   reject="avsigclip", project=no, outtype="real", offsets="none", 
	   masktype="none", maskvalue=0., blank=0., scale="exposure", 
	   zero="none", weight="exposure",
	   statsec="", expname="inttime", lthreshold=INDEF, 
	   hthreshold=INDEF, nlow=1,
	   nhigh=1, mclip=yes, lsigma=2., hsigma=2., rdnoise="9", gain="6.5",
	   sigscale=0.1, pclip=-0.5, grow=0)

#	if (cb == yes ) {

#	   imarith(imfinraw//ii,
#	     "/",iflatsmooth//ii,
#	     imfin//ii, title="", divzero=0., hparams="", 
#	     pixtype="real",calctype="real", verbose=no, noact=no)

#	}

#	if (cb == no ) {

	   imcopy (imfinraw//ii,imfin//ii,verbose=no)

#	}

# compute location of SAOIMAGE image display and display combine spectrum

	  disp_cen=disp_cen+ratio

	  if ( disp_cen >= (1.-ratio/2) ) {
	    disp_cen=1.-ratio/2
	  }

	   if ( cc == yes ) {

	   display (imfinraw//ii,
	   1, erase=no, border_erase=no, select_frame=yes, repeat=no, 
	   fill=yes, zscale=yes, contrast=0.25, zrange=no, nsample_line=5,
	   xcenter=disp_cen, ycenter=0.5, xsize=1., ysize=1., xmag=1., 
	   ymag=1., order=0, z1=900., z2=3000.,
	   ztrans="linear", lutfile="")

	   }

# redo background computation if not happy

#	if ( answer == yes ) {

#	del(zeropos//"*", ver-)
#	del(insertpos, ver-)
#	del(pixcor//"*", ver-)

#	imdel(imfinraw//ii)
#	imdel(imindiv//ii)
#	imdel(imagebias//"*.imh")
#	imdel(imageflatfin//"*.imh")
#	imdel(imagefin//"*.imh")
#	imdel(imagerest//"*.imh")

#	disp_cen=ratio

#	del (corfile, ver-)
#	del (corfile2, ver-)
#	del (listflatfin, ver-)

#	ij=1

#	goto redosky

#	}


# clean up

	    imdel(imagebias//ii//"*.imh")
	    imdel(imageflat//ii//"*.imh")
	    imdel(imageflatfin//"*.imh")
	    imdel(imageflaten//ii//"*.imh")
	    imdel(ims//ii//"*.imh")
	    imdel(imb//ii//"*.imh")
	    imdel(imf//ii//"*.imh")
	    imdel(lambda//ii//"*.imh")
	    imdel(imagerest//"*.imh")
	    imdel(iflatsmooth//ii//"*.imh")
	    imdel(imfinraw//ii//"*.imh")
	    del(corfile, ver-)
	    del(corfile2, ver-)
	    del(imgfile, ver-)
	    del(zeropos//"*", ver-)
	    del(insertpos//"*", ver-)
	    del (listflatfin//"*", ver-)

	imdel("tt"//"*.imh")
	del("ttins"//"*", ver-)
	del("ttzero"//"*", ver-)

	if ( cc == yes ) {

	  if ( anext == yes ) {

	    aaa = 1

	    dir(imfin//ii//"*.imh")
	    dir(imagecal//ii//"*.imh")

 	  } else if ( previous == yes ) {

	    print(previous)

	    bbb = 1

	    imdel(imfin//ii//"*.imh")
	    imdel(imagecal//ii//"*.imh")
	    imdel(imindiv//ii//"*.imh")

	    disp_cen=ratio

	    goto toto

	  } 

	  print(aaa,bbb)

	  if ( aaa != 1 ) {
	    if ( bbb != 1 ) {
	      print("get out")
	      break
	    }
	  }

	}

slit:

	ii+=1
	ij=1
#	print(ii)

	}

	end
	

