# TASK multiextract.cl
# MULTI-SLIT DATA PROCESSING
# FOR EACH SLIT THIS PROCEDURE DOES THE FOLLOWING:
# - DISPLAY INDIVIDUAL CORRECTED 2D SPECTRA 
# - DISPLAY FINAL 2D SPECTRUM
# - COMPUTE WAVELENGTH SOLUTION
# - EXTRACT 1D SPECTRA
# - CALIBRATE IN LAMBDA
# - CALIBRATE IN FLUX
# OLIVIER LE FEVRE 
# version 1: 18 Dec 1992

procedure multiextract (spec_ima,slit_nums)

string spec_ima {prompt="Spectrum root name"}
int slit_nums {1, prompt="Slit start"}
int slit_nume {1, prompt="Slit end"}
string lambda_ref {prompt="Image reference wavelength calib"}
string flux_cal {prompt="sensitivity image for flux calib"}
bool cd {no, prompt="wavelength calib. interactively ?", mode="q"}
bool ce {yes, prompt="extract object(s) interactively (>1 per aperture = yes) ?", mode="q"}
bool cf {yes, prompt="Do you want to flux calibrate ?", mode="q"}
bool cg {yes, prompt="smooth before extraction ?", mode="q"}
bool anext = yes {prompt="Go to next object ?", mode="q"}
bool previous = yes {prompt="Redo previous object ?", mode="q"}

begin
#	bool anext,previous
	int ij = 1
	int aaa = 0
	int bbb = 0
	int xmax, ymax, zmax, ii
	string imageback, imagefin, imspec2d,imspec1d,imfina,imagecal
	string imfinaa,imagecala,imsens, imfin,refcal
	real disp_cen, ratio

	imagefin=spec_ima
	refcal=lambda_ref
	imsens=flux_cal
	i=slit_nums

	for ( ii=i; ii<=slit_nume; ii+=1 ) {

toto:
	print(ii)
	aaa=0
	bbb=0

	imfin=imagefin
	imspec1d=imagefin//ii//".0001"
	imageback=imagefin//"back"//ii
	imfina=imfin//ii//".000*.imh"
	imagecal=imfin//"cal"
	imagecala=imagecal//ii//".0001"
	imfinaa = imfina//".000*.imh"

	imgets (imageback,"i_naxis1", value="")
	xmax=int(imgets.value)

	imgets (imageback,"i_naxis2", value="")
	ymax=int(imgets.value)

	imgets (imageback,"i_naxis3", value="")
	zmax=int(imgets.value)

	print(xmax,ymax,zmax)

	ratio=xmax*1./(ymax*1.)
	disp_cen=ratio/2.

	ij = 1

	for (ij=1; ij <= zmax; ij+=1) {

	  print(ij,disp_cen)

	  if ( disp_cen >= 1 ) {
	    disp_cen=1.-ratio/2
	  }

	   if ( ij == 1 ) { 

	   display (imageback//"[*,*,"//ij//"]",
	   1, erase=yes, border_erase=no, select_frame=yes, repeat=no, 
	   fill=yes, zscale=yes, contrast=0.25, zrange=no, nsample_line=5, 
	   xcenter=disp_cen, ycenter=0.5, xsize=1., ysize=1., xmag=1., 
	   ymag=1., order=0, z1=900., z2=3000.,
	   ztrans="linear", lutfile="")

	   } else {

	   display (imageback//"[*,*,"//ij//"]",
	   1, erase=no, border_erase=no, select_frame=yes, repeat=no, 
	   fill=yes, zscale=yes, contrast=0.25, zrange=no, nsample_line=5, 
	   xcenter=disp_cen, ycenter=0.5, xsize=1., ysize=1., xmag=1., 
	   ymag=1., order=0, z1=900., z2=3000.,
	   ztrans="linear", lutfile="")

	   }

	  disp_cen=disp_cen+ratio

	}

	if ( disp_cen >= 1 ) {
	  disp_cen=1.-ratio/2
	 }

	display (imfin//ii,
	1, erase=no, border_erase=no, select_frame=yes, repeat=no, 
	fill=yes, zscale=yes, contrast=0.25, zrange=no, nsample_line=5, 
	xcenter=disp_cen, ycenter=0.5, xsize=1., ysize=1., xmag=1., 
	ymag=1., order=0, z1=900., z2=3000.,
	ztrans="linear", lutfile="")

# find aperture ll calib

	print("apfind for lambda cal.")

	apfind (imagecal//ii, 1, references="", interactive=yes, 
	 find=yes, recenter=no, resize=no, edit=yes,
	 line=INDEF, nsum=1, minsep=5., maxsep=1000., order="increasing")
 
#  extract wavelength calibration spectrum

	print("apsum ll calib")

	apsum (imagecal//ii,
	 output="", format="onedspec", references="", 
	 profiles="", interactive=no,
	 find=no, recenter=no, resize=no, edit=no, trace=no, fittrace=yes,
	 extract=yes, extras=no, review=no, line=INDEF, nsum=100, 
	 background="none",
	 weights="none", pfit="fit1d", clean=no, skybox=1, saturation=INDEF,
	 readnoise="0.", gain="1.", lsigma=4., usigma=4., nsubaps=1)

# identify features in calibration spectrum

	print("reidentify")

	if ( cd == no ) {

	   reidentify (refcal,
	   imagecala, "yes", interactive="no", section="*", 
	   newaps=yes, override=yes,
	   refit=yes, trace=no, step=10, nsum=10, shift=INDEF, nlost=3, 
	   cradius=15.,
	   threshold=500., addfeatures=no,
#           coordlist="linelists$idhenear.dat", 
           coordlist="linelist",
	   match=10.,
	   maxfeatures=10000, minsep=2., database="database", 
	   logfiles="STDOUT,logfile",
	   plotfile="", verbose=no, graphics="stdgraph", cursor="")

	}

	else {

	   reidentify (refcal,
	   imagecala, "yes", interactive="yes", section="*", 
	   newaps=yes, override=yes,
	   refit=yes, trace=no, step=10, nsum=10, shift=INDEF, nlost=3, 
	   cradius=15.,
	   threshold=500., addfeatures=no, 
#           coordlist="linelists$idhenear.dat", 
           coordlist="linelist",	   
	   match=10.,
	   maxfeatures=500, minsep=2., database="database", 
	   logfiles="STDOUT,logfile",
	   plotfile="", verbose=no, graphics="stdgraph", cursor="")

	}
 
	if ( ce == no ) {

# find aperture object, only 1 aperture allowed in auto mode
# for more than 1 aperture use manual extraction ( ce==yes )

	   apfind (imfin//ii,
	   1, references="", interactive=no, find=yes, recenter=no, 
	   resize=no, edit=no,
	   line=INDEF, nsum=400, minsep=5., maxsep=1000., order="increasing")

#  smooth with gaussian sigma=0.7 if wanted

	   if ( cg == yes ) { 

	   print("smooth 2D spectrum")

	     gauss (imfin//ii, imfin//ii, 0.7, ratio=1., theta=0., 
	     nsigma=6., bilinear=yes,
	     boundary="nearest", constant=0.)

	   }
 
#  extract object spectrum

	print("apsum objet")

	   apsum (imfin//ii,
	   output="", format="onedspec", references="", 
	   profiles="", interactive=no,
	   find=no, recenter=no, resize=no, edit=no, trace=yes, fittrace=yes,
	   extract=yes, extras=no, review=no, line=INDEF, nsum=300, 
	   background="none", weights="variance", pfit="fit1d", clean=yes,
	   skybox=1, saturation=INDEF,
	   readnoise="9.", gain="6.5", lsigma=4., usigma=4., nsubaps=1)

	}

	else {

#  smooth with gaussian sigma=0.7 if wanted

	if ( cg == yes ) { 

	print("smooth 2D spectrum")

	  gauss (imfin//ii, imfin//ii, 0.7, ratio=1., theta=0., 
	  nsigma=6., bilinear=yes,
	  boundary="nearest", constant=0.)

	}
 
#  extract object spectrum

	print("apsum objet")

	   apsum (imfin//ii,
	   output="", format="onedspec", references="", 
	   profiles="", interactive=yes,
	   find=no, recenter=no, resize=no, edit=yes, trace=yes, fittrace=yes,
	   extract=yes, extras=no, review=yes, line=INDEF, nsum=300, 
	   background="none", weights="variance", pfit="fit1d", clean=yes,
	   skybox=1, saturation=INDEF,
	   readnoise="9.", gain="6.5", lsigma=4., usigma=4., nsubaps=1)

	}

# assign reference spectrum and wavelength calibrate object spectrum 
# do it for all 1D spectra extracted from current 2D spectrum

	print("refspec")

	   refspectra (imfina,
	   "yes", references=imagecala, apertures="", refaps="", ignoreaps=yes,
	   select="interp", sort="", group="", time=yes, timewrap=17., 
	   override=no,
	   confirm=yes, assign=yes, logfiles="STDOUT,logfile", verbose=no)
 
	print("dispcor")

	   dispcor (imfina,
	   imfina, linearize=yes, database="database", table="", 
	   w1=INDEF, w2=INDEF,
#	   dw=INDEF, nw=INDEF, log=no, flux=yes, samedisp=no, 
	   dw=INDEF, nw=INDEF, flux=yes, samedisp=no, 
	   global=no, ignoreaps=yes,
	   confirm=no, listonly=no, verbose=yes, logfile="")


	if ( cf == yes ) {

	print("fluxcalib")

           hedit (imfina, "DISPAXIS", "1", add=yes, delete=no,
           verify=no, show=yes, update=yes)

	   fluxcalib (imfina,
	   imfina, imsens, fnu=yes, exposure="inttime")

	}
 

# splot final

	   splot (imfina)	   

# clean up

	  if ( anext == yes ) {

	    aaa = 1

	    dir(imfin//ii//"*.imh")
	    dir(imagecal//ii//"*.imh")

 	  } else if ( previous == yes ) {

	    print(previous)

	    bbb = 1

	    disp_cen=ratio

	    goto toto

	    } 

#	  print(aaa,bbb)

	  if ( aaa != 1 ) {
	    if ( bbb != 1 ) {
	      print("get out")
	      break
	    }
	  }

	ij=1
#	print(ii)


	}

	end
	


