# TASK multisky.cl
# MULTI-SLIT DATA PROCESSING
# FOR EACH SLIT THIS PROCEDURE DOES THE FOLLOWING:
# - DISPLAY INDIVIDUAL CORRECTED 2D SPECTRA 
# - DISPLAY FINAL 2D SPECTRUM
# - COMPUTE WAVELENGTH SOLUTION
# - EXTRACT 1D SPECTRA
# - CALIBRATE IN LAMBDA
# - CALIBRATE IN FLUX
# OLIVIER LE FEVRE 
# version 1: 18 Dec 1992

procedure multiextract (spec_ima,slit_pos,slit_nums,slit_nume)

string spec_raw {prompt="Raw spectra image"}
string spec_ima {prompt="Spectrum root name"}
string slit_pos {prompt="File w/ slit positions"}
int slit_nums {1, prompt="Slit start"}
int slit_nume {1, prompt="Slit end"}
real lambda0 {prompt="Zero deviation wavelength"}
real lambda1 {prompt="Start wavelength"}
real lambda2 {prompt="End wavelength"}
real disper {prompt="Dispersion in A/pix"}
bool anext = yes {prompt="Go to next object ?", mode="q"}
bool previous = yes {prompt="Redo previous object ?", mode="q"}

struct *crtlist {mode="hl"}

begin
#	bool anext,previous
	int ii=1
	int ij = 1
	int aaa = 0
	int bbb = 0
	int xmin, ymin, xmax, ymax, zmax, xcalmin, xcalmax, lmax,xima
	int x,y,l,ylow,yhigh
	int xc1[3],xc2[3],yc1[3],yc2[3],number,num,xi,yi,ymid,yavg,ysup,yrep
	int x1,x2,x3,x4,x5,x6, orderzero, os1, os2, os3, oss
	int ll[1000],of,os
	int xca1,xca2,yca1,yca2,xcb1,xcb2,ycb1,ycb2,xcc1,xcc2,ycc1,ycc2
	int xcor,ycor,xins,yins,xflat1,xflat2
	string imageback, imagefin, imspec2d,imspec1d,imfina,imagecal
	string imfinaa,imagecala,imsens, imfin,refcal, specraw,list
	string imagezero,imagezerocor,func, funczero, funczero1,funczero2
	string funczero3, idnum, filezerocor,specext
	real disp_cen, ratio

	imagefin=spec_ima
	specraw=spec_raw
	i=slit_nums
	list=slit_pos
	crtlist=list

# start loop on slit positions file

	while (fscan (crtlist, x, y, l) != EOF)  {

	#, xflat1, xflat2, of,number,
	#xca1,xca2,yca1,yca2,funczero1,os1,
	#xcb1,xcb2,ycb1,ycb2,funczero2,os2,
	#xcc1,xcc2,ycc1,ycc2,funczero3,os3,
	#x1,x2,x3,x4,x5,x6, func,os)

	if ( ii < slit_nums || ii > slit_nume ) goto slit
toto:
	print (ii)
	print (x,y,l)

	aaa=0
	bbb=0

	imfin=imagefin//"sky"
	imspec1d=imagefin//ii//".0001"
	imageback=imagefin//"back"//ii
	imfina=imfin//ii//".000*.imh"
	imagecal=imagefin//"cal"
	imagecala=imagecal//ii//".0001"
	imfinaa = imfina//".000*.imh"

	   x=x
	   xmin = x-l/2
	   xmax = x+l/2-1
	   xcalmin = x-2
	   xcalmax = x+2
	   lmax=l-1

# get input images pixel size in Y axis

	   imgets (specraw,"i_naxis2", value="")
	   ymax=int(imgets.value)

	   print(ymax)

# compute ymin and ymax

	  ylow=y-(lambda0-lambda1)/disper
	  yhigh=y+(lambda2-lambda0)/disper

	  if (ylow < 1 ) {
	    ymin=1
	  } else {
	    ymin=ylow
	  }

	  if ( yhigh > ymax ) {
	    ymax=ymax
	  } else {
	    ymax=yhigh
	  }

	  if (ymax > 2008 ) {
	    ymax=2008
	  }

	  print("ymin,ymax")
	  print(ymin,ymax)

	  imcopy(specraw//"["//xmin//":"//xmax//","//ymin//":"//ymax//"]",
	   imfin//ii, verbose=no)

# find aperture raw spectrum

	print("apfind for raw spectrum")

	apfind (imfin//ii, 1, references="", interactive=yes, 
	 find=yes, recenter=no, resize=no, edit=yes,
	 line=INDEF, nsum=1, minsep=5., maxsep=1000., order="increasing")
 
#  extract wavelength calibration spectrum

	print("apsum raw spectrum")

	apsum (imfin//ii,
	 output="", format="onedspec", references="", 
	 profiles="", interactive=no,
	 find=no, recenter=no, resize=no, edit=no, trace=no, fittrace=yes,
	 extract=yes, extras=no, review=no, line=INDEF, nsum=100, 
	 background="none",
	 weights="none", pfit="fit1d", clean=no, skybox=1, saturation=INDEF,
	 readnoise="0.", gain="1.", lsigma=4., usigma=4., nsubaps=1)

# assign reference spectrum and wavelength calibrate object spectrum 
# do it for all 1D spectra extracted from current 2D spectrum

	print("refspec")

	   refspectra (imfina,
	   "yes", references=imagecala, apertures="", refaps="", ignoreaps=yes,
	   select="interp", sort="", group="", time=yes, timewrap=17., 
	   override=no,
	   confirm=yes, assign=yes, logfiles="STDOUT,logfile", verbose=no)
 
	print("dispcor")

	   dispcor (imfina,
	   imfina, linearize=yes, database="database", table="", 
	   w1=INDEF, w2=INDEF,
#	   dw=INDEF, nw=INDEF, log=no, flux=yes, samedisp=no, 
	   dw=INDEF, nw=INDEF, flux=yes, samedisp=no, 
	   global=no, ignoreaps=yes,
	   confirm=no, listonly=no, verbose=yes, logfile="")


# splot final

	   splot (imfina)	   

# clean up

	  if ( anext == yes ) {

	    aaa = 1

 	  } else if ( previous == yes ) {

	    print(previous)

	    bbb = 1

	    disp_cen=ratio
	    goto toto

	    } 

#	  print(aaa,bbb)

	  if ( aaa != 1 ) {
	    if ( bbb != 1 ) {
	      print("get out")
	      break
	    }
	  }

slit:
	ij=1
	ii+=1
#	print(ii)

	}

	end
	


